# @(#) $Id: email.bro 73 2009-02-14 02:12:48Z leres $ (LBL)
#
# Copyright (c) 2009
#      The Regents of the University of California.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that: (1) source code distributions
# retain the above copyright notice and this paragraph in its entirety, (2)
# distributions including binary code include the above copyright notice and
# this paragraph in its entirety in the documentation or other materials
# provided with the distribution, and (3) all advertising materials mentioning
# features or use of this software display the following acknowledgement:
# ``This product includes software developed by the University of California,
# Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
# the University nor the names of its contributors may be used to endorse
# or promote products derived from this software without specific prior
# written permission.
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#

module email;

export {
	## Tunables
	# How long to wait before emailing multiple message lines
	const emailbatchdelta: interval = 60 mins &redef;

	# Where to send notification
	const watcher: string = "root" &redef;

	# Subject line (fmt() called with this and gethostname())
	const emailsubj: string = "%s bro report" &redef;

	# Public interface
	global email: function(line: string);
}

## Globals
global emailmsg: string = "";
global emailtime: time = 0;

# Email a report using the argument as the message body
function
deliver(msg: string)
{
	local temp = fmt("/tmp/email.%d", getpid());
	# don't map "\n" -> "^J"
	local f: file = open(temp) &raw_output;

@ifdef ( notdef )
	# XXX &raw_output doesn't work
	print f, msg;
@else
	local lines = split(msg, /\^J/);
	local i = 0;
	for (i in lines)
		print f, lines[i];
@endif
	close(f);
	local subj = fmt(emailsubj, gethostname());
	system(fmt("mail -s '%s' %s < %s ; rm %s", subj, watcher, temp, temp));
}

# Handler to flush any pending messages out via an email message
event
flush()
{
	emailtime = 0;
	if (emailmsg != "") {
		email(emailmsg);
		emailmsg = "";
	}
}

# queue up a message line
function
email(line: string)
{
	local now = current_time();

	if (emailmsg == "")
		emailmsg = line;
	else
		emailmsg = cat(emailmsg, "\n", line);

	#
	# If nothing has happened recently, email immediately.
	# Otherwise wait some amount of time before emailing a report.
	#
	if (emailtime == 0) {
		deliver(emailmsg);
		emailmsg = "";
		emailtime = now;
		schedule emailbatchdelta {
			flush()
		};
	}
}

#
# Flush any pending lines
#
# Note: bro_done() is called after persistent state is saved
#
event
bro_done()
{
	if (emailmsg != "")
		deliver(emailmsg);
}
