MON            (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)
DAY            ([ 0-3][0-9])
TIME           ([ 0-2][0-9]\:[0-6][0-9]\:[0-6][0-9])
TS             {MON}" "{DAY}" "{TIME}
O              ([0-9]|[0-9][0-9]|[0-1][0-9][0-9]|2[0-4][0-9]|25[0-5])
N              ([0-9]+)
IP             {O}((\.{O}){3})
LOCALHOST      (127\.0\.0\.1)
FIRST          [A-Za-z0-9]
MIDDLE         [-.A-Za-z0-9]
LAST           [A-Za-z0-9]
HN             ({FIRST}{MIDDLE}*{LAST}|{FIRST}{MIDDLE}*|{MIDDLE}*{LAST})
NSP            [^ \n]
OC             (opened|closed)

    #include <sys/types.h>
    #include <sys/socket.h>
    #include <sys/stat.h>
    #include <sys/time.h>

    #include <netinet/in.h>

    #include <arpa/inet.h>
    #include <arpa/nameser.h>

    #include <broccoli.h>
    #include <ctype.h>
    #include <errno.h>
    #include <netdb.h>
    #include <signal.h>
    #include <stdarg.h>
    #include <stdio.h>
    #include <string.h>
    #include <sysexits.h>
    #include <syslog.h>
    #include <time.h>
    #include <unistd.h>

    #include "setsignal.h"
    #include "version.h"

    #undef yywrap
    #define yywrap() (1)
    #ifdef FLEX_SCANNER
    #define YY_NO_UNPUT
    #endif

    #undef YY_INPUT
    #define YY_INPUT(buf, result, max_size) { \
	    result = tail(buf, max_size); \
    }

    int yylex(void);
    inline int tail(char *, int);
    void lg(int, const char *, ...)
	__attribute__ ((format (printf, 2, 3)));
    const char *name(const char *);
    const char *chain2(const char *, int);
    const char *chain3(const char *, int);
    const char *chainhack(const char *, int);
    void pix(const char *, const char *, const char *, const char *,
	const char *, const char *);
    void ssh(const char *, const char *, const char *, const char *,
	const char *, const char *);
    inline void copystr(char *, size_t, const char *, size_t);

    int i;
    char junkchar = '\0';
    const char *p;
    int debug = 0;
    u_int lineno = 1;
    char myip[32] = "";
    const char *what;
    char ts[32] = "";
    char src[512] = "";
    char dst[512] = "";
    char inside[64] = "";
    char outside[64] = "";
    char proto[64] = "";
    char user[128] = "";
    char method[64] = "";
    const char *file = "/var/log/syslog";
    const char e_ping[] = "syslog2bro::ping";
    const char e_pong[] = "syslog2bro::pong";
    const char e_ssh_failed[] = "syslog2bro::ssh_fail_login";
    const char e_ssh_invalid[] = "syslog2bro::ssh_invalid";
    const char e_ssh_success[] = "syslog2bro::ssh_login";
    const char e_pix_deny[] = "syslog2bro::pix_deny";

%x ident eat meat junk
%x sshd junksshd sshdgetuser sshdgetmethod sshdgetsrcip
%x pixstart pixproto pixoutside pixgetoutside pixinside pixgetinside

%%

	/* "Jan  9 06:26:45" */

<INITIAL>^{TS}" "	{
			/* Don't save trailing blank */
			copystr(ts, sizeof(ts), yytext, yyleng - 1);
			BEGIN(ident);
			}

	/* Lines that don't start with a timestamp are junk */

<INITIAL>^.		{
			if (debug > 1)
				fprintf(stderr, "%s:%d JUNK: \"%s",
				    file, lineno, yytext);
			BEGIN(junk);
			}

	/* There are many ways to pick up the syslog host IP */

<ident>({IP}|{HN})" "	{
			/* Don't save trailing blank */
			copystr(dst, sizeof(dst), yytext, yyleng - 1);
			BEGIN(meat);
			}

<ident>{IP}"/"{IP}"/"{IP}" " {
			/* Don't save trailing blank */
			p = chain3(yytext, yyleng - 1);
			copystr(dst, sizeof(dst), p, strlen(p));
			BEGIN(meat);
			}

<ident>{HN}"/"{IP}"/"{IP}" " {
			/* Don't save trailing blank */
			p = chain3(yytext, yyleng - 1);
			copystr(dst, sizeof(dst), p, strlen(p));
			BEGIN(meat);
			}

<ident>{LOCALHOST}"/"{IP}" " {
			/* Save 2nd IP in chain */
			p = strchr(yytext, '/');
			if (p == NULL)
				abort();
			++p;
			/* Don't save trailing blank */
			copystr(dst, sizeof(dst), p, strlen(p) - 1);
			BEGIN(meat);
			}

<ident>({IP}|{HN})"/"{LOCALHOST}" " {
			/* Save 1st ip address in chain */
			p = strchr(yytext, '/');
			if (p == NULL)
				abort();
			i = p - yytext;
			copystr(dst, sizeof(dst), yytext, i);
			BEGIN(meat);
			}

<ident>{IP}"/"{IP}" "	{
			/* Not IP/127.0.0.1 or 127.0.0.1/IP so grab 1st addr */
			p = strchr(yytext, '/');
			if (p == NULL)
				abort();
			i = p - yytext;
			copystr(dst, sizeof(dst), yytext, i);
			BEGIN(meat);
			}

<ident>{HN}"/"{IP}" "	{
			/* Since this isn't FOO/127.0.0.1, grab the 2nd IP */
			p = strchr(yytext, '/');
			if (p == NULL)
				abort();
			++p;
			/* Don't save trailing blank */
			copystr(dst, sizeof(dst), p, strlen(p) - 1);
			BEGIN(meat);
			}

<ident>{NSP}+"/"{IP}" "	{
			/* (Yech) */
			/* Don't save trailing blank */
			p = chainhack(yytext, yyleng - 1);
			copystr(dst, sizeof(dst), p, strlen(p));
			BEGIN(meat);
			}

<junk>.*		{
			if (debug > 1)
				fprintf(stderr, "%s\"\n", yytext);
			}

 /****************************************************************************/

	/* "sshd[16045]: " */

<meat>sshd{NSP}+" "	BEGIN(sshd);

	/* [ID 800047 auth.info] */

<sshd>"[ID "{NSP}+" "{NSP}+"] "	/* Eat syslog-ng junk */
<sshd>log:" "			/* Eat syslog-ng junk */
<sshd>error:" "			/* Eat uninteresting preamble */
<sshd>PAM:" "			/* Eat uninteresting preamble */

<sshd>Failed" "		{
			what = e_ssh_failed;
			strcpy(method, "password");
			BEGIN(sshdgetmethod);
			}

	/* Illegal user n3ssus from 131.243.64.203 */
	/* Invalid user shepherd from 128.3.30.16 */

<sshd>(Illegal|Invalid)" user " {
			/*
			 * Don't use ssh_failed here since these
			 * are usually accompanied by a login failure
			 */
			what = e_ssh_invalid;
			strcpy(method, "password");
			BEGIN(sshdgetuser);
			}

	/* Password authentication failed for user dave from ife2.lbl.gov. */

<sshd>"Password authentication failed for user " {
			what = e_ssh_failed;
			strcpy(method, "password");
			BEGIN(sshdgetuser);
			}

	/* Authentication failure for illegal user kirk from 149.99.20.238 */
	/* (Ahoy, Captian!) */

<sshd>"Authentication failure for illegal user " {
			what = e_ssh_failed;
			strcpy(method, "password");
			BEGIN(sshdgetuser);
			}

	/* Authentication failure for root from 208.40.197.182 */

<sshd>"Authentication failure for " {
			what = e_ssh_failed;
			strcpy(method, "password");
			BEGIN(sshdgetuser);
			}

	/* authentication error for illegal user angel from 194.192.15.69 */

<sshd>"authentication error for illegal user " {
			what = e_ssh_failed;
			strcpy(method, "password");
			BEGIN(sshdgetuser);
			}

	/* Accepted publickey for leres from 131.243.2.202 port 52159 ssh2 */

<sshd>Accepted" "	{
			what = e_ssh_success;
			BEGIN(sshdgetmethod);
			}

	/* Eat these to avoid cluttering syslog */

<sshd>"Did not receive identification"			BEGIN(eat);
<sshd>"session "{OC}" for user"				BEGIN(eat);
<sshd>"(pam_unix) session "{OC}				BEGIN(eat);
<sshd>"pam_unix(sshd:session): session "{OC}		BEGIN(eat);
<sshd>"pam_unix(ssh:auth):"				BEGIN(eat);
<sshd>"roles pam_sm_authenticate"			BEGIN(eat);
<sshd>"ROOT LOGIN"					BEGIN(eat);
<sshd>"Closing connection"				BEGIN(eat);
<sshd>[Cc]onnect					BEGIN(eat);
<sshd>RSA						BEGIN(eat);
<sshd>Protocol						BEGIN(eat);
<sshd>Kerberos						BEGIN(eat);
<sshd>Postponed						BEGIN(eat);
<sshd>"subsystem request"				BEGIN(eat);
<sshd>"Password authentication for "{NSP}+" failed."	BEGIN(eat);
<sshd>"Bad protocol version identification"		BEGIN(eat);
<sshd>"Generating new "[0-9]+" bit RSA key."		BEGIN(eat);

    /* Log/eat ssh lines we don't understad */

<sshd>.			{
			junkchar = *yytext;
			BEGIN(junksshd);
			}

<junksshd>.*		{
			/* Log these so we know when to tweak the patterns */
			lg(LOG_DEBUG, "%s:%d SSHD JUNK: \"%c%s\"",
			    file, lineno, junkchar, yytext);
			}

<sshdgetmethod>{NSP}+" for " {
			copystr(method, sizeof(method), yytext, yyleng - 5);
			BEGIN(sshdgetuser);
			}

<sshdgetmethod>{NSP}+" for invalid user " {
			copystr(method, sizeof(method), yytext, yyleng - 18);
			BEGIN(sshdgetuser);
			}

<sshdgetuser>{NSP}+" from "	{
			i = sizeof(" from ") - 1;
			copystr(user, sizeof(user), yytext, yyleng - i);
			BEGIN(sshdgetsrcip);
			}

<sshdgetsrcip>::ffff:	/* Eat IPV6 prefix */

<sshdgetsrcip>{IP}	{
			copystr(src, sizeof(src), yytext, yyleng);
			ssh(what, ts, src, dst, user, method);
			BEGIN(eat);
			}

<sshdgetsrcip>{HN}	{
			/* Yech. */
			copystr(src, sizeof(src), yytext, yyleng);
			ssh(what, ts, src, dst, user, method);
			BEGIN(eat);
			}

 /****************************************************************************/

	/* "%PIX-4-106023: " */

<meat>%PIX-{N}-{N}:" "	BEGIN(pixstart);
<meat>%ASA-{N}-{N}:" "	BEGIN(pixstart);

	/* "Deny tcp src" */

<pixstart>.*Deny" "	{
			what = e_pix_deny;
			BEGIN(pixproto);
			}

<pixproto>(icmp|tcp|udp)	{
			copystr(proto, sizeof(proto), yytext, yyleng);
			BEGIN(pixoutside);
			}

	/* "outside:10.0.0.21" */

<pixoutside>[^:]*:	BEGIN(pixgetoutside);

<pixgetoutside>{HN}	{
			copystr(outside, sizeof(outside), yytext, yyleng);
			BEGIN(pixinside);
			}

	/* dst nets:syslog/514 */

<pixinside>[^:]*:	BEGIN(pixgetinside);

<pixgetinside>{HN}	{
			copystr(inside, sizeof(inside), yytext, yyleng);
			pix(what, ts, dst, outside, inside, proto);
			BEGIN(eat);
			}

 /****************************************************************************/

<meat>.			BEGIN(eat);

<eat>.*			/* eat */

<*>.			{
			/* XXX shouldn't see many of these */
			if (debug)
				fprintf(stderr, "{%s:%d '%s'}\n",
				    file, lineno, yytext);
			}

<*>\n			{
			++lineno;
			what = "?";
			ts[0] = '\0';
			src[0] = '\0';
			dst[0] = '\0';
			user[0] = '\0';
			method[0] = '\0';
			inside[0] = '\0';
			outside[0] = '\0';
			proto[0] = '\0';
			junkchar = '\0';
			BEGIN(INITIAL);
			}

%%

/*
 * Copyright (c) 2008, 2009
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
static const char copyright[] =
    "@(#) Copyright (c) 2008, 2009\n\
The Regents of the University of California.  All rights reserved.\n";
static const char rcsid[] =
    "@(#) $Id: syslog2bro.l 118 2009-10-15 23:06:37Z leres $ (LBL)";
#endif

char *prog;

int targc;
char **targv;

extern char *optarg;
extern int optind, opterr;

/* Forwards */
int main(int, char **);
void connect2bro(void);
RETSIGTYPE die(int);
void getmyip(char *, size_t);
struct timeval *parsetime(const char *);
void ping(void);
void pong(BroConn *, void *, double *);
void service(long);
int text2inaddr(const char *, const char *, struct in_addr *);
time_t timestamp(const char *);
const char *tsstr(void);
void usage(void) __attribute__((noreturn));

/* Globals */
int bflag;
int Bflag;
int text;
int sawpong;
struct stat sbuf;
const char *pidfile;
const char localhost[] = "127.0.0.1";
struct sockaddr_in brosin;
int s = -1;
BroConn *bc;
int foreground;

/* Broccoli */
const char *addr = "127.0.0.1";
const char *port = "47757";
static int timeout = 10;			/* select timeout */
u_int32_t cookie;

int
main(argc, argv)
	int argc;
	char **argv;
{
	int n;
	char *cp, *ep;
	int op;
	FILE *f;

	if (argv[0] == NULL)
		prog = "syslog2bro";
	else if ((cp = strrchr(argv[0], '/')) != NULL)
		prog = cp + 1;
	else
		prog = argv[0];

	openlog(prog, 0, LOG_DAEMON);

	opterr = 0;
	while ((op = getopt(argc, argv, "bBdFta:f:p:P:")) != EOF)
		switch (op) {

		case 'a':
			addr = optarg;
			break;

		case 'b':
			++bflag;
			break;

		case 'B':
			++Bflag;
			break;

		case 'd':
			++debug;
			break;

		case 'f':
			file = optarg;
			break;

		case 'F':
			++foreground;
			break;

		case 'p':
			pidfile = optarg;
			break;

		case 'P':
			port = optarg;
			break;

		case 't':
			++text;
			++foreground;
			break;

		default:
			usage();
		}

	if (argc != optind)
		usage();
	if (bflag && Bflag)
		bflag = 0;

	if (!foreground && *file != '/') {
		fprintf(stderr, "%s: %s is not a full path name\n", prog, file);
		exit(EX_USAGE);
	}

	lg(LOG_INFO, "version %s starting", version);

	memset(&brosin, 0, sizeof(brosin));
#ifdef HAVE_SOCKADDR_SA_LEN
	brosin.sin_len = sizeof(brosin);
#endif
	brosin.sin_family = AF_INET;

	i = strtol(port, &ep, 10);
	if (*ep != '\0' || i < 1 || i > 0xffff) {
		fprintf(stderr, "%s: bad port value (%s)\n", prog, port);
		exit(EX_USAGE);
	}
	brosin.sin_port = htons(i);

	/* "*" -> 0.0.0.0 else decode string address */
	if (strcmp(addr, "*") != 0 &&
	    inet_pton(brosin.sin_family, addr, &brosin.sin_addr) < 0) {
		fprintf(stderr, "%s: bad address value (%s): %s\n",
		    prog, addr, strerror(errno));
		exit(EX_USAGE);
	}

	if (!foreground && daemon(0, 1) < 0) {
		lg(LOG_ERR, "daemon: %s", strerror(errno));
		exit(EX_OSERR);
	}

	if (!text)
		connect2bro();

	timeout = timeout;

	getmyip(myip, sizeof(myip));
	(void)setsignal(SIGINT, die);
	(void)setsignal(SIGTERM, die);

	yyin = fopen(file, "r");
	if (yyin == NULL) {
		fprintf(stderr, "%s: %s: %s\n",
		    prog, file, strerror(errno));
		exit(1);
	}
	if (fstat(fileno(yyin), &sbuf) < 0) {
		fprintf(stderr, "%s: fstat %s: %s\n",
		    prog, file, strerror(errno));
		exit(1);
	}
	if (!Bflag && !bflag && lseek(fileno(yyin), 0, SEEK_END) < 0) {
		fprintf(stderr, "%s: lseek %s: %s\n",
		    prog, file, strerror(errno));
		exit(1);
	}

	if (pidfile != NULL) {
		f = fopen(pidfile, "w");
		if (f == NULL) {
			fprintf(stderr, "%s: fopen %s: %s\n",
			    prog, pidfile, strerror(errno));
			exit(1);
		}
		fprintf(f, "%u\n", getpid());
		fclose(f);
	}

	/* Process input */
	yylex();

	if (bc != NULL) {
		/* Ping Bro and wait for him to catch up */
		sawpong = 0;

		/* These seem to get lost sometimes? Try more than once  */
		n = 0;
		do {
			ping();
			service(1);
		} while (++n < 5 && !sawpong);
		if (!sawpong)
			lg(LOG_ERR, "WARNING: Bro didn't answer ping");
		bro_conn_delete(bc);
		close(s);
	}

	exit(0);
}

/* Handles HN/IP and IP/IP */
const char *
chain2(const char *str, int len)
{
	int i, j;
	const char *cp;
	static char tstr[256];

	cp = strchr(str, '/');
	if (cp == NULL) {
		lg(LOG_ERR, "%s:%d: missing '/': %s", file, lineno, str);
		return ("?");
	}
	i = cp - str;
	if (i > sizeof(tstr) - 1)
		i = sizeof(tstr) - 1;
	++cp;
	j = len - i;
	--j;
	if (j > sizeof(tstr) - 1)
		j = sizeof(tstr) - 1;

	/* Return the 1st part if the 2nd is localhost */
	if (strncmp(localhost, cp, j) == 0)
		strncpy(tstr, str, i);
	else
		strncpy(tstr, cp, j);
	return (tstr);
}

/* Handles HN/IP/IP and IP/IP/IP (strip off the last IP and invoke chain2) */
const char *
chain3(const char *str, int len)
{
	const char *cp;

	cp = strrchr(str, '/');
	if (cp == NULL) {
		return ("?");
	}
	len = cp - str;
	return (chain2(str, len));
}

/*
 * Deal with cruft like:
 *
 *  "Oct  5 01:55:02 /Applications/Retrospect/131.243.10.24 6.1/Retro ..."
 *  "Oct  5 00:00:02 \tRev./128.3.28.153 2 LSI, Inc. 1064 found."
 */
const char *
chainhack(const char *str, int len)
{
	const char *cp;
	char temp[256];

	if (debug > 2)
		fprintf(stderr, "%s:%d: chainhack \"%.*s\"\n",
		    file, lineno, len, str);
	cp = strrchr(str, '/');
	(void)snprintf(temp, sizeof(temp), "?%s", cp);
	return (chain2(temp, strlen(temp)));
}

void
connect2bro(void)
{
	int n, secs;

	for (n = 0;; ++n) {
		/* We need a new socket for each connect() failure */
		s = socket(PF_INET, SOCK_STREAM, 0);
		if (s < 0) {
			lg(LOG_ERR, "socket: %s", strerror(errno));
			exit(EX_OSERR);
		}
		if (connect(s,
		    (struct sockaddr *)&brosin, sizeof(brosin)) >= 0)
			break;
		lg(LOG_ERR, "connect(%s.%d): %s",
		    inet_ntoa(brosin.sin_addr), ntohs(brosin.sin_port),
		    strerror(errno));
		close(s);
		if (n > 60 * 10)
			secs = 60 * 10;
		else if (n > 60)
			secs = 60;
		else if (n > 10)
			secs = 10;
		else
			secs = 1;
		sleep(secs);
	}

	bc = bro_conn_new_socket(s, BRO_CFLAG_RECONNECT);
	if (bc == NULL) {
		lg(LOG_ERR, "bro_conn_new_socket to %s:%d failed",
		    inet_ntoa(brosin.sin_addr), ntohs(brosin.sin_port));
		exit(EX_UNAVAILABLE);
	}

	/* Set the connection class */
	bro_conn_set_class(bc, "syslog2bro");

	/* Register handler */
	bro_event_registry_add(bc, e_pong, (BroEventFunc)pong, NULL);

	while (!bro_conn_connect(bc)) {
		lg(LOG_DEBUG, "Failed to connect; retrying...");
		sleep(1);
	}

	lg(LOG_INFO, "connected to %s:%s", addr, port);
}

inline void
copystr(char *buf, size_t size, const char *str, size_t len)
{
	int i;

	i = len;
	if (i >= size)
		i = size - 1;
	strncpy(buf, str, i);
	buf[i] = '\0';
}

RETSIGTYPE
die(int signo)
{
	char buf[32];

	switch (signo) {

	case SIGINT:
		strcpy(buf, "INT");
		break;

	case SIGTERM:
		strcpy(buf, "TERM");
		break;

	default:
		(void)snprintf(buf, sizeof(buf), "%d", signo);
		break;
	}
	fflush(stdout);
	fprintf(stderr, "%s: exiting on signal %s\n", prog, buf);
	lg(LOG_NOTICE, "exiting on signal %s", buf);
	exit(1);

}

void
getmyip(char *myip, size_t size)
{
	char host[256];

	if (gethostname(host, sizeof(host)) < 0)
		strcpy(myip, "?");
	else
		snprintf(myip, size, "%s", name(host));
}

void
lg(int pri, const char *fmt, ...)
{
	va_list ap;

	if (debug) {
		fputs(tsstr(), stderr);
		putc(' ', stderr);
		va_start(ap, fmt);
		(void)vfprintf(stderr, fmt, ap);
		putc('\n', stderr);
		va_end(ap);
	} else {
		va_start(ap, fmt);
		vsyslog(pri, fmt, ap);
		va_end(ap);
	}
}

/* Important that resolv.conf has a "search" config for all possible domains */
const char *
name(const char *host)
{
	struct hostent *hp;
	struct in_addr in;

	hp = gethostbyname(host);
	if (hp == NULL) {
		if (debug > 1)
			fprintf(stderr, "%s:%d: gethostbyname(%s) failed\n",
			    file, lineno, host);
		return ("?");
	}

	if (hp->h_length != NS_INADDRSZ) {
		fprintf(stderr, "%s:%d: hp->h_length != NS_INADDRSZ (%d)\n",
		    file, lineno, hp->h_length);
		return ("?");
	}

	if (hp->h_addrtype != AF_INET) {
		fprintf(stderr, "%s:%d: hp->h_addrtype != AF_INET (%d)\n",
		    file, lineno, hp->h_addrtype);
		return ("?");
	}

	memmove(&in.s_addr, hp->h_addr_list[0], NS_INADDRSZ);
	return (inet_ntoa(in));
}

struct timeval *
parsetime(const char *str)
{
	char *cp;
	time_t now;
	struct tm tm, *tp;
	static struct timeval tv;

	if (gettimeofday(&tv, NULL) < 0) {
		lg(LOG_ERR, "parsetime: gettimeofday: %s", strerror(errno));
		exit(EX_OSERR);
	}
	now = tv.tv_sec;
	tp = localtime(&now);
	tm = *tp;
	tp = &tm;

	/* "Oct 20 14:40:57" */
	cp = strptime(str, "%b %d %T", tp);
	if (cp == NULL) {
		lg(LOG_ERR, "parsetime: strptime failed: %s", strerror(errno));
		return (NULL);
	}
	if (*cp != '\0') {
		lg(LOG_ERR, "parsetime: bad string: %s", str);
		return (NULL);
	}
	tv.tv_sec = mktime(tp);
	tv.tv_usec = 0;
	return (&tv);
}

void
ping()
{
	double now;
	BroEvent *ev;

	if (bc == NULL)
		return;

	ev = bro_event_new(e_ping);
	if (ev == NULL) {
		lg(LOG_ERR, "ping: bro_event_new failed");
		return;
	}

	/* timestamp */
	now = bro_util_current_time();
	if (!bro_event_add_val(ev, BRO_TYPE_TIME, NULL, &now)) {
		lg(LOG_ERR, "ping: bro_record_add_val(ts) failed");
		return;
	}

	if (!bro_event_send(bc, ev))
		lg(LOG_DEBUG, "ping: FAILED to send event");
	bro_event_free(ev);
}

void
pong(BroConn *bc, void *data, double *thenp)
{
	double now;

	if (debug) {
		now = bro_util_current_time();
		lg(LOG_ERR, "pong: delta %.6f", now - *thenp);
	}
	++sawpong;
}

void
pix(const char *eventstr, const char *ts, const char *dst, const char *outside,
    const char *inside, const char *proto)
{
	double x;
	BroEvent *ev;
	BroString str;
	struct in_addr adst, aoutside, ainside;
	struct timeval *tp;
	char tdst[32], toutside[32];

	/* ts */
	tp = parsetime(ts);
	if (tp == NULL) {
		lg(LOG_ERR, "pix: bogus ts time \"%s\"", ts);
		return;
	}
	x = bro_util_timeval_to_double(tp);

	/* "dst" */
	if (!text2inaddr("pix: bogus dst", dst, &adst))
		return;

	/* "outside" */
	if (!text2inaddr("pix: bogus outside", outside, &aoutside))
		return;

	/* "inside" */
	if (!text2inaddr("pix: bogus inside", inside, &ainside))
		return;

	if (bc == NULL) {
		copystr(tdst, sizeof(tdst), inet_ntoa(adst), sizeof(tdst));
		copystr(toutside, sizeof(toutside), inet_ntoa(aoutside),
		    sizeof(toutside));
		printf("%s double=%.0f addr=%s addr=%s addr=%s string=%s\n",
		    eventstr, x, tdst, toutside, inet_ntoa(ainside), proto);
		return;
	}

	ev = bro_event_new(eventstr);
	if (ev == NULL) {
		lg(LOG_ERR, "pix: bro_event_new failed");
		return;
	}

	/* "ts" */
	if (!bro_event_add_val(ev, BRO_TYPE_TIME, NULL, &x)) {
		lg(LOG_ERR, "pix: bro_record_add_val(ts) failed");
		return;
	}

	/* "dst" */
	if (!bro_event_add_val(ev, BRO_TYPE_IPADDR, NULL, &adst)) {
		lg(LOG_ERR, "pix: bro_record_add_val(dst) failed");
		return;
	}

	/* "outside" */
	if (!bro_event_add_val(ev, BRO_TYPE_IPADDR, NULL, &aoutside)) {
		lg(LOG_ERR, "pix: bro_record_add_val(outside) failed");
		return;
	}

	/* "inside" */
	if (!bro_event_add_val(ev, BRO_TYPE_IPADDR, NULL, &ainside)) {
		lg(LOG_ERR, "pix: bro_record_add_val(inside) failed");
		return;
	}

	/* "proto" */
	bro_string_init(&str);
	bro_string_set(&str, proto);
	if (!bro_event_add_val(ev, BRO_TYPE_STRING, NULL, &str)) {
		lg(LOG_ERR, "pix: bro_record_add_val(proto) failed");
		return;
	}
	bro_string_cleanup(&str);

	if (!bro_event_send(bc, ev))
		lg(LOG_DEBUG, "pix: FAILED to send event");
	bro_event_free(ev);

}

/* Bro service routine */
void
service(long secs)
{
	int n, nfds;
	fd_set readfds;
	struct timeval tmo;

	/* Calculate fdset */
	FD_ZERO(&readfds);
	FD_SET(s, &readfds);
	nfds = s + 1;

	/* Set timeout (0 seconds means poll) */
	tmo.tv_sec = secs;
	tmo.tv_usec = 0;

	n = select(nfds, &readfds, NULL, NULL, &tmo);

	if (n < 0) {
		/* Don't choke if we get ptraced */
		if (errno == EINTR)
			return;
		lg(LOG_ERR, "select: %s", strerror(errno));
		exit(EX_OSERR);
	}
	if (n > 0 && FD_ISSET(s, &readfds)) {
		if (debug)
			lg(LOG_DEBUG, "calling bro_conn_process_input()");
		if (!bro_conn_process_input(bc) && !bro_conn_alive(bc)) {
			lg(LOG_INFO, "server exited; attempting to reconnect");
			connect2bro();
		}
	}
}

void
ssh(const char *eventstr, const char *ts, const char *src, const char *dst,
    const char *user, const char *method)
{
	double x;
	BroEvent *ev;
	BroString str;
	struct in_addr asrc, adst;
	struct timeval *tp;
	char tstr[32];

	/* ts */
	tp = parsetime(ts);
	if (tp == NULL) {
		lg(LOG_ERR, "ssh: bogus ts time \"%s\"", ts);
		return;
	}
	x = bro_util_timeval_to_double(tp);

	/* "src" */
	if (!text2inaddr("ssh: bogus src", src, &asrc))
		return;

	/* "dst" */
	if (!text2inaddr("ssh: bogus dst", dst, &adst))
		return;

	if (bc == NULL) {
		copystr(tstr, sizeof(tstr), inet_ntoa(asrc), sizeof(tstr));
		printf("%s double=%.0f addr=%s addr=%s string=%s string=%s\n",
		    eventstr, x, tstr, inet_ntoa(adst), user, method);
		return;
	}

	ev = bro_event_new(eventstr);
	if (ev == NULL) {
		lg(LOG_ERR, "ssh: bro_event_new failed");
		return;
	}

	/* "ts" */
	if (!bro_event_add_val(ev, BRO_TYPE_TIME, NULL, &x)) {
		lg(LOG_ERR, "ssh: bro_record_add_val(ts) failed");
		return;
	}

	/* "src" */
	if (!bro_event_add_val(ev, BRO_TYPE_IPADDR, NULL, &asrc)) {
		lg(LOG_ERR, "ssh: bro_record_add_val(src) failed");
		return;
	}

	/* "dst" */
	if (!bro_event_add_val(ev, BRO_TYPE_IPADDR, NULL, &adst)) {
		lg(LOG_ERR, "ssh: bro_record_add_val(dst) failed");
		return;
	}

	/* "user" */
	bro_string_init(&str);
	bro_string_set(&str, user);
	if (!bro_event_add_val(ev, BRO_TYPE_STRING, NULL, &str)) {
		lg(LOG_ERR, "ssh: bro_record_add_val(user) failed");
		return;
	}
	bro_string_cleanup(&str);

	/* "method" */
	bro_string_init(&str);
	bro_string_set(&str, method);
	if (!bro_event_add_val(ev, BRO_TYPE_STRING, NULL, &str)) {
		lg(LOG_ERR, "ssh: bro_record_add_val(method) failed");
		return;
	}
	bro_string_cleanup(&str);

	if (!bro_event_send(bc, ev))
		lg(LOG_DEBUG, "ssh: FAILED to send event");
	bro_event_free(ev);

}

/* Implement "tail -F" semantics */
inline int
tail(char *buf, int max_size)
{
	static int lastresult = 1 << 31;
	int result;
	struct stat sbuf2;

	/* Try to read more data per sys call */
	if (lastresult < 512)
		(void)usleep(250000);

	while ((result = read(fileno(yyin), buf, max_size)) <= 0) {
		if (result < 0) {
			fprintf(stderr, "%s: %s: read: %s\n",
			    prog, file, strerror(errno));
			exit(1);
		}

		/* Broccoli processing */
		if (bc != NULL)
			service(0);

		/* If we got here, we must have seen EOF. Optionally bail */
		if (bflag)
			return (YY_NULL);
		(void)usleep(250000);

		/* If file missing, wait for it to be recreated */
		if (stat(file, &sbuf2) < 0)
			continue;

		if (sbuf2.st_ino != sbuf.st_ino ||
		    sbuf2.st_dev != sbuf.st_dev) {
			yyin = freopen(file, "r", yyin);
			if (yyin != NULL) {
				lg(LOG_INFO, "reopened %s", file);
				memmove(&sbuf, &sbuf2, sizeof(sbuf));
			}
		}
	}
	lastresult = result;
	return (result);
}

int
text2inaddr(const char *what, const char *str, struct in_addr *ap)
{
	const char *cp;

	if (inet_aton(str, ap) == 0) {
		/* XXX should convert directly to uint32 */
		cp = name(str);
		if (inet_aton(cp, ap) == 0) {
			lg(LOG_ERR, "%s (%s)", what, cp);
			return (0);
		}
	}
	return (1);
}

/* Convert timestamp string */
time_t
timestamp(const char *ts)
{
	time_t now;
	struct tm tm;
	struct tm *tp;
	static int year = 0;
	static int month = -1;

	/* Populate get current year */
	memset(&tm, 0, sizeof(tm));
	if (strptime(ts, "%b %d %H:%M:%S", &tm) == NULL)
		return (0);

	/* Determine current year if the month changed */
	if (month != tm.tm_mon) {
		month = tm.tm_mon;
		now = time(NULL);
		tp = localtime(&now);
		if (tp == NULL) {
			fprintf(stderr, "localtime chewed it\n");
			exit (1);
		}
		year = tp->tm_year;
	}
	tm.tm_year = year;

	return (mktime(&tm));
}

const char *
tsstr()
{
	struct timeval tv;
	static char buf[32];

	if (gettimeofday(&tv, NULL) < 0) {
		lg(LOG_ERR, "getts: gettimeofday: %s", strerror(errno));
		exit(1);
	}

	snprintf(buf, sizeof(buf), "%ld.%06ld",
	    (long)tv.tv_sec, (long)tv.tv_usec);
	return (buf);
}

void
usage()
{

	(void)fprintf(stderr, "Version %s\n", version);
	(void)fprintf(stderr,
	    "usage: %s [-bBdFt] [-p pidfile] [-a addr] [-P port] [-f file]"
#ifdef notdef
	    " [port:]host"
#endif
	    "\n", prog);
	exit(1);
}
